@@ -381,9 +381,8 @@ class BaseDatabaseWrapper(object):
         self.ensure_connection()
 
         start_transaction_under_autocommit = (
-            force_begin_transaction_with_broken_autocommit
-            and not autocommit
-            and self.features.autocommits_when_autocommit_is_off
+            force_begin_transaction_with_broken_autocommit and not autocommit and
+            self.features.autocommits_when_autocommit_is_off
         )
 
         if start_transaction_under_autocommit:
@@ -514,13 +513,14 @@ class BaseDatabaseWrapper(object):
         authorized to be shared between threads (via the `allow_thread_sharing`
         property). Raises an exception if the validation fails.
         """
-        if not (self.allow_thread_sharing
-                or self._thread_ident == thread.get_ident()):
-            raise DatabaseError("DatabaseWrapper objects created in a "
+        if not (self.allow_thread_sharing or self._thread_ident == thread.get_ident()):
+            raise DatabaseError(
+                "DatabaseWrapper objects created in a "
                 "thread can only be used in that same thread. The object "
                 "with alias '%s' was created in thread id %s and this is "
                 "thread id %s."
-                % (self.alias, self._thread_ident, thread.get_ident()))
+                % (self.alias, self._thread_ident, thread.get_ident())
+            )
 
     # ##### Miscellaneous #####
 
