@@ -11,9 +11,13 @@ from itertools import chain
 
 from django.conf import settings
 from django.forms.utils import flatatt, to_current_timezone
+from django.templatetags.static import static
 from django.utils import datetime_safe, formats, six
 from django.utils.datastructures import MultiValueDict
 from django.utils.dates import MONTHS
+from django.utils.deprecation import (
+    RemovedInDjango20Warning, RenameMethodsBase,
+)
 from django.utils.encoding import (
     force_str, force_text, python_2_unicode_compatible,
 )
@@ -21,7 +25,6 @@ from django.utils.formats import get_format
 from django.utils.html import conditional_escape, format_html, html_safe
 from django.utils.safestring import mark_safe
 from django.utils.six.moves import range
-from django.utils.six.moves.urllib.parse import urljoin
 from django.utils.translation import ugettext_lazy
 
 __all__ = (
@@ -77,16 +80,15 @@ class Media(object):
             ) for path in self._css[medium]
         ] for medium in media])
 
-    def absolute_path(self, path, prefix=None):
+    def absolute_path(self, path):
+        """
+        Given a relative or absolute path to a static asset, return an absolute
+        path. An absolute path will be returned unchanged while a relative path
+        will be passed to django.templatetags.static.static().
+        """
         if path.startswith(('http://', 'https://', '/')):
             return path
-        if prefix is None:
-            if settings.STATIC_URL is None:
-                # backwards compatibility
-                prefix = settings.MEDIA_URL
-            else:
-                prefix = settings.STATIC_URL
-        return urljoin(prefix, path)
+        return static(path)
 
     def __getitem__(self, name):
         "Returns a Media object that only contains media of the given type"
@@ -148,8 +150,7 @@ class MediaDefiningClass(type):
     Metaclass for classes that can have media definitions.
     """
     def __new__(mcs, name, bases, attrs):
-        new_class = (super(MediaDefiningClass, mcs)
-            .__new__(mcs, name, bases, attrs))
+        new_class = super(MediaDefiningClass, mcs).__new__(mcs, name, bases, attrs)
 
         if 'media' not in attrs:
             new_class.media = media_property(new_class)
@@ -176,7 +177,13 @@ class SubWidget(object):
         return self.parent_widget.render(*args)
 
 
-class Widget(six.with_metaclass(MediaDefiningClass)):
+class RenameWidgetMethods(MediaDefiningClass, RenameMethodsBase):
+    renamed_methods = (
+        ('_format_value', 'format_value', RemovedInDjango20Warning),
+    )
+
+
+class Widget(six.with_metaclass(RenameWidgetMethods)):
     needs_multipart_form = False  # Determines does this widget need multipart form
     is_localized = False
     is_required = False
@@ -230,6 +237,9 @@ class Widget(six.with_metaclass(MediaDefiningClass)):
         """
         return data.get(name)
 
+    def value_omitted_from_data(self, data, files, name):
+        return name not in data
+
     def id_for_label(self, id_):
         """
         Returns the HTML ID attribute of this Widget for use by a <label>,
@@ -242,6 +252,9 @@ class Widget(six.with_metaclass(MediaDefiningClass)):
         """
         return id_
 
+    def use_required_attribute(self, initial):
+        return not self.is_hidden
+
 
 class Input(Widget):
     """
@@ -250,7 +263,7 @@ class Input(Widget):
     """
     input_type = None  # Subclasses must define this.
 
-    def _format_value(self, value):
+    def format_value(self, value):
         if self.is_localized:
             return formats.localize_input(value)
         return value
@@ -261,7 +274,7 @@ class Input(Widget):
         final_attrs = self.build_attrs(attrs, type=self.input_type, name=name)
         if value != '':
             # Only add the 'value' attribute if a value is non-empty.
-            final_attrs['value'] = force_text(self._format_value(value))
+            final_attrs['value'] = force_text(self.format_value(value))
         return format_html('<input{} />', flatatt(final_attrs))
 
 
@@ -308,12 +321,7 @@ class MultipleHiddenInput(HiddenInput):
     A widget that handles <input type="hidden"> for fields that have a list
     of values.
     """
-    def __init__(self, attrs=None, choices=()):
-        super(MultipleHiddenInput, self).__init__(attrs)
-        # choices can be any iterable
-        self.choices = choices
-
-    def render(self, name, value, attrs=None, choices=()):
+    def render(self, name, value, attrs=None):
         if value is None:
             value = []
         final_attrs = self.build_attrs(attrs, type=self.input_type, name=name)
@@ -345,6 +353,9 @@ class FileInput(Input):
         "File widgets take data from FILES, not POST"
         return files.get(name)
 
+    def value_omitted_from_data(self, data, files, name):
+        return name not in files
+
 
 FILE_INPUT_CONTRADICTION = object()
 
@@ -378,7 +389,7 @@ class ClearableFileInput(FileInput):
         """
         Return whether value is considered to be initial value.
         """
-        return bool(value and hasattr(value, 'url'))
+        return bool(value and getattr(value, 'url', False))
 
     def get_template_substitution_values(self, value):
         """
@@ -426,6 +437,9 @@ class ClearableFileInput(FileInput):
             return False
         return upload
 
+    def use_required_attribute(self, initial):
+        return super(ClearableFileInput, self).use_required_attribute(initial) and not initial
+
 
 class Textarea(Widget):
     def __init__(self, attrs=None):
@@ -439,9 +453,7 @@ class Textarea(Widget):
         if value is None:
             value = ''
         final_attrs = self.build_attrs(attrs, name=name)
-        return format_html('<textarea{}>\r\n{}</textarea>',
-                           flatatt(final_attrs),
-                           force_text(value))
+        return format_html('<textarea{}>\r\n{}</textarea>', flatatt(final_attrs), force_text(value))
 
 
 class DateTimeBaseInput(TextInput):
@@ -452,9 +464,8 @@ class DateTimeBaseInput(TextInput):
         super(DateTimeBaseInput, self).__init__(attrs)
         self.format = format if format else None
 
-    def _format_value(self, value):
-        return formats.localize_input(value,
-            self.format or formats.get_format(self.format_key)[0])
+    def format_value(self, value):
+        return formats.localize_input(value, self.format or formats.get_format(self.format_key)[0])
 
 
 class DateInput(DateTimeBaseInput):
@@ -502,6 +513,11 @@ class CheckboxInput(Widget):
             value = values.get(value.lower(), value)
         return bool(value)
 
+    def value_omitted_from_data(self, data, files, name):
+        # HTML checkboxes don't appear in POST data if not checked, so it's
+        # never known if the value is actually omitted.
+        return False
+
 
 class Select(Widget):
     allow_multiple_selected = False
@@ -520,12 +536,12 @@ class Select(Widget):
         memo[id(self)] = obj
         return obj
 
-    def render(self, name, value, attrs=None, choices=()):
+    def render(self, name, value, attrs=None):
         if value is None:
             value = ''
         final_attrs = self.build_attrs(attrs, name=name)
         output = [format_html('<select{}>', flatatt(final_attrs))]
-        options = self.render_options(choices, [value])
+        options = self.render_options([value])
         if options:
             output.append(options)
         output.append('</select>')
@@ -542,16 +558,13 @@ class Select(Widget):
                 selected_choices.remove(option_value)
         else:
             selected_html = ''
-        return format_html('<option value="{}"{}>{}</option>',
-                           option_value,
-                           selected_html,
-                           force_text(option_label))
+        return format_html('<option value="{}"{}>{}</option>', option_value, selected_html, force_text(option_label))
 
-    def render_options(self, choices, selected_choices):
+    def render_options(self, selected_choices):
         # Normalize to strings.
         selected_choices = set(force_text(v) for v in selected_choices)
         output = []
-        for option_value, option_label in chain(self.choices, choices):
+        for option_value, option_label in self.choices:
             if isinstance(option_label, (list, tuple)):
                 output.append(format_html('<optgroup label="{}">', force_text(option_value)))
                 for option in option_label:
@@ -567,37 +580,41 @@ class NullBooleanSelect(Select):
     A Select Widget intended to be used with NullBooleanField.
     """
     def __init__(self, attrs=None):
-        choices = (('1', ugettext_lazy('Unknown')),
-                   ('2', ugettext_lazy('Yes')),
-                   ('3', ugettext_lazy('No')))
+        choices = (
+            ('1', ugettext_lazy('Unknown')),
+            ('2', ugettext_lazy('Yes')),
+            ('3', ugettext_lazy('No')),
+        )
         super(NullBooleanSelect, self).__init__(attrs, choices)
 
-    def render(self, name, value, attrs=None, choices=()):
+    def render(self, name, value, attrs=None):
         try:
             value = {True: '2', False: '3', '2': '2', '3': '3'}[value]
         except KeyError:
             value = '1'
-        return super(NullBooleanSelect, self).render(name, value, attrs, choices)
+        return super(NullBooleanSelect, self).render(name, value, attrs)
 
     def value_from_datadict(self, data, files, name):
         value = data.get(name)
-        return {'2': True,
-                True: True,
-                'True': True,
-                '3': False,
-                'False': False,
-                False: False}.get(value)
+        return {
+            '2': True,
+            True: True,
+            'True': True,
+            '3': False,
+            'False': False,
+            False: False,
+        }.get(value)
 
 
 class SelectMultiple(Select):
     allow_multiple_selected = True
 
-    def render(self, name, value, attrs=None, choices=()):
+    def render(self, name, value, attrs=None):
         if value is None:
             value = []
         final_attrs = self.build_attrs(attrs, name=name)
         output = [format_html('<select multiple="multiple"{}>', flatatt(final_attrs))]
-        options = self.render_options(choices, value)
+        options = self.render_options(value)
         if options:
             output.append(options)
         output.append('</select>')
@@ -631,7 +648,7 @@ class ChoiceInput(SubWidget):
     def __str__(self):
         return self.render()
 
-    def render(self, name=None, value=None, attrs=None, choices=()):
+    def render(self, name=None, value=None, attrs=None):
         if self.id_for_label:
             label_for = format_html(' for="{}"', self.id_for_label)
         else:
@@ -693,8 +710,11 @@ class ChoiceFieldRenderer(object):
         self.choices = choices
 
     def __getitem__(self, idx):
-        choice = self.choices[idx]  # Let the IndexError propagate
-        return self.choice_input_class(self.name, self.value, self.attrs.copy(), choice, idx)
+        return list(self)[idx]
+
+    def __iter__(self):
+        for idx, choice in enumerate(self.choices):
+            yield self.choice_input_class(self.name, self.value, self.attrs.copy(), choice, idx)
 
     def __str__(self):
         return self.render()
@@ -720,16 +740,18 @@ class ChoiceFieldRenderer(object):
                     choices=choice_label,
                 )
                 sub_ul_renderer.choice_input_class = self.choice_input_class
-                output.append(format_html(self.inner_html, choice_value=choice_value,
-                                          sub_widgets=sub_ul_renderer.render()))
+                output.append(format_html(
+                    self.inner_html, choice_value=choice_value,
+                    sub_widgets=sub_ul_renderer.render(),
+                ))
             else:
-                w = self.choice_input_class(self.name, self.value,
-                                            self.attrs.copy(), choice, i)
-                output.append(format_html(self.inner_html,
-                                          choice_value=force_text(w), sub_widgets=''))
-        return format_html(self.outer_html,
-                           id_attr=format_html(' id="{}"', id_) if id_ else '',
-                           content=mark_safe('\n'.join(output)))
+                w = self.choice_input_class(self.name, self.value, self.attrs.copy(), choice, i)
+                output.append(format_html(self.inner_html, choice_value=force_text(w), sub_widgets=''))
+        return format_html(
+            self.outer_html,
+            id_attr=format_html(' id="{}"', id_) if id_ else '',
+            content=mark_safe('\n'.join(output)),
+        )
 
 
 class RadioFieldRenderer(ChoiceFieldRenderer):
@@ -751,20 +773,19 @@ class RendererMixin(object):
             self.renderer = renderer
         super(RendererMixin, self).__init__(*args, **kwargs)
 
-    def subwidgets(self, name, value, attrs=None, choices=()):
-        for widget in self.get_renderer(name, value, attrs, choices):
+    def subwidgets(self, name, value, attrs=None):
+        for widget in self.get_renderer(name, value, attrs):
             yield widget
 
-    def get_renderer(self, name, value, attrs=None, choices=()):
+    def get_renderer(self, name, value, attrs=None):
         """Returns an instance of the renderer."""
         if value is None:
             value = self._empty_value
         final_attrs = self.build_attrs(attrs)
-        choices = list(chain(self.choices, choices))
-        return self.renderer(name, value, final_attrs, choices)
+        return self.renderer(name, value, final_attrs, self.choices)
 
-    def render(self, name, value, attrs=None, choices=()):
-        return self.get_renderer(name, value, attrs, choices).render()
+    def render(self, name, value, attrs=None):
+        return self.get_renderer(name, value, attrs).render()
 
     def id_for_label(self, id_):
         # Widgets using this RendererMixin are made of a collection of
@@ -786,6 +807,16 @@ class CheckboxSelectMultiple(RendererMixin, SelectMultiple):
     renderer = CheckboxFieldRenderer
     _empty_value = []
 
+    def use_required_attribute(self, initial):
+        # Don't use the 'required' attribute because browser validation would
+        # require all checkboxes to be checked instead of at least one.
+        return False
+
+    def value_omitted_from_data(self, data, files, name):
+        # HTML checkboxes don't appear in POST data if not checked, so it's
+        # never known if the value is actually omitted.
+        return False
+
 
 class MultiWidget(Widget):
     """
@@ -852,6 +883,12 @@ class MultiWidget(Widget):
     def value_from_datadict(self, data, files, name):
         return [widget.value_from_datadict(data, files, name + '_%s' % i) for i, widget in enumerate(self.widgets)]
 
+    def value_omitted_from_data(self, data, files, name):
+        return all(
+            widget.value_omitted_from_data(data, files, name + '_%s' % i)
+            for i, widget in enumerate(self.widgets)
+        )
+
     def format_output(self, rendered_widgets):
         """
         Given a list of rendered widgets (as strings), returns a Unicode string
@@ -895,8 +932,10 @@ class SplitDateTimeWidget(MultiWidget):
     supports_microseconds = False
 
     def __init__(self, attrs=None, date_format=None, time_format=None):
-        widgets = (DateInput(attrs=attrs, format=date_format),
-                   TimeInput(attrs=attrs, format=time_format))
+        widgets = (
+            DateInput(attrs=attrs, format=date_format),
+            TimeInput(attrs=attrs, format=time_format),
+        )
         super(SplitDateTimeWidget, self).__init__(widgets, attrs)
 
     def decompress(self, value):
@@ -1035,6 +1074,12 @@ class SelectDateWidget(Widget):
                 return '%s-%s-%s' % (y, m, d)
         return data.get(name)
 
+    def value_omitted_from_data(self, data, files, name):
+        return not any(
+            ('{}_{}'.format(name, interval) in data)
+            for interval in ('year', 'month', 'day')
+        )
+
     def create_select(self, name, field, value, val, choices, none_value):
         if 'id' in self.attrs:
             id_ = self.attrs['id']
